@@ -19,47 +19,7 @@
 #include <qpdf/Pipeline.hh>
 #include <qpdf/Pl_Buffer.hh>
 
-class TokenFilter : public QPDFObjectHandle::TokenFilter {
-public:
-    using QPDFObjectHandle::TokenFilter::TokenFilter;
-    virtual ~TokenFilter() = default;
-    using Token            = QPDFTokenizer::Token;
-
-    void handleToken(Token const &token) override
-    {
-        py::object result = this->handle_token(token);
-        if (result.is_none())
-            return;
-        try {
-            if (py::hasattr(result, "__iter__")) {
-                for (auto item : result) {
-                    const auto returned_token = item.cast<Token>();
-                    this->writeToken(returned_token);
-                }
-            } else {
-                const auto returned_token = result.cast<Token>();
-                this->writeToken(returned_token);
-            }
-        } catch (const py::cast_error &e) {
-            throw py::type_error("returned object that is not a token");
-        }
-    }
-
-    virtual py::object handle_token(Token const &token) = 0;
-};
-
-class TokenFilterTrampoline : public TokenFilter {
-public:
-    using TokenFilter::TokenFilter;
-    using Token = QPDFTokenizer::Token;
-
-    py::object handle_token(Token const &token) override
-    {
-        PYBIND11_OVERLOAD_PURE(py::object, TokenFilter, handle_token, token);
-    }
-};
-
-size_t page_index(QPDF &owner, QPDFObjectHandle page)
+py::size_t page_index(QPDF &owner, QPDFObjectHandle page)
 {
     if (&owner != page.getOwningQPDF())
         throw py::value_error("Page is not in this Pdf");
@@ -90,9 +50,18 @@ void init_page(py::module_ &m)
 {
     py::class_<QPDFPageObjectHelper>(m, "Page")
         .def(py::init<QPDFObjectHandle &>())
+        .def(py::init([](QPDFPageObjectHelper &poh) {
+            return QPDFPageObjectHelper(poh.getObjectHandle());
+        }))
+        .def("__hash__",
+            [](QPDFPageObjectHelper &poh) {
+                throw py::type_error("Can't hash mutable object");
+            })
         .def_property_readonly(
             "obj",
-            [](QPDFPageObjectHelper &poh) { return poh.getObjectHandle(); },
+            [](QPDFPageObjectHelper &poh) -> QPDFObjectHandle {
+                return poh.getObjectHandle();
+            },
             R"~~~(
                 Get the underlying :class:`pikepdf.Object`.
             )~~~")
@@ -137,39 +106,29 @@ void init_page(py::module_ &m)
                 confuse some software.
             )~~~")
         .def(
-            "contents_add",
+            "_contents_add",
             [](QPDFPageObjectHelper &poh, QPDFObjectHandle &contents, bool prepend) {
                 return poh.addPageContents(contents, prepend);
             },
             py::arg("contents"),
-            // py::kw_only(), // TODO in 3.x
+            py::kw_only(),
             py::arg("prepend") = false,
-            py::keep_alive<1, 2>(),
-            R"~~~(
-                Append or prepend to an existing page's content stream using an existing stream object.
-
-                .. versionadded:: 2.14
-            )~~~")
+            py::keep_alive<1, 2>())
         .def(
-            "contents_add",
+            "_contents_add",
             [](QPDFPageObjectHelper &poh, py::bytes contents, bool prepend) {
                 auto q = poh.getObjectHandle().getOwningQPDF();
                 if (!q) {
                     // LCOV_EXCL_START
                     throw std::logic_error("QPDFPageObjectHelper not attached to QPDF");
-                    // LCOV_EXCL_END
+                    // LCOV_EXCL_STOP
                 }
                 auto stream = QPDFObjectHandle::newStream(q, contents);
                 return poh.addPageContents(stream, prepend);
             },
             py::arg("contents"),
             py::kw_only(),
-            py::arg("prepend") = false,
-            R"~~~(
-                Append or prepend to an existing page's content stream from bytes.
-
-                .. versionadded:: 2.14
-            )~~~")
+            py::arg("prepend") = false)
         .def("remove_unreferenced_resources",
             &QPDFPageObjectHelper::removeUnreferencedResources,
             R"~~~(
@@ -214,12 +173,12 @@ void init_page(py::module_ &m)
                 bool invert_transformations,
                 bool allow_shrink,
                 bool allow_expand) -> py::bytes {
-                return poh.placeFormXObject(formx,
+                return py::bytes(poh.placeFormXObject(formx,
                     name.getName(),
                     rect,
                     invert_transformations,
                     allow_shrink,
-                    allow_expand);
+                    allow_expand));
             },
             py::arg("formx"),
             py::arg("name"),
@@ -252,7 +211,8 @@ void init_page(py::module_ &m)
             )~~~")
         .def(
             "get_filtered_contents",
-            [](QPDFPageObjectHelper &poh, TokenFilter &tf) {
+            [](QPDFPageObjectHelper &poh,
+                QPDFObjectHandle::TokenFilter &tf) -> py::bytes {
                 Pl_Buffer pl_buffer("filter_page");
                 poh.filterPageContents(&tf, &pl_buffer);
 
@@ -276,7 +236,7 @@ void init_page(py::module_ &m)
                 To modify the content stream, use :meth:`pikepdf.Page.add_content_token_filter`.
 
                 Returns:
-                    bytes: the modified content stream
+                    The modified content stream.
             )~~~")
         .def(
             "add_content_token_filter",
@@ -379,88 +339,4 @@ void init_page(py::module_ &m)
                     Returns the ordinary page number if no special rules for page
                     numbers are defined.
             )~~~");
-
-    py::enum_<QPDFTokenizer::token_type_e>(m, "TokenType")
-        .value("bad", QPDFTokenizer::token_type_e::tt_bad)
-        .value("array_close", QPDFTokenizer::token_type_e::tt_array_close)
-        .value("array_open", QPDFTokenizer::token_type_e::tt_array_open)
-        .value("brace_close", QPDFTokenizer::token_type_e::tt_brace_close)
-        .value("brace_open", QPDFTokenizer::token_type_e::tt_brace_open)
-        .value("dict_close", QPDFTokenizer::token_type_e::tt_dict_close)
-        .value("dict_open", QPDFTokenizer::token_type_e::tt_dict_open)
-        .value("integer", QPDFTokenizer::token_type_e::tt_integer)
-        .value("name", QPDFTokenizer::token_type_e::tt_name)
-        .value("real", QPDFTokenizer::token_type_e::tt_real)
-        .value("string", QPDFTokenizer::token_type_e::tt_string)
-        .value("null", QPDFTokenizer::token_type_e::tt_null)
-        .value("bool", QPDFTokenizer::token_type_e::tt_bool)
-        .value("word", QPDFTokenizer::token_type_e::tt_word)
-        .value("eof", QPDFTokenizer::token_type_e::tt_eof)
-        .value("space", QPDFTokenizer::token_type_e::tt_space)
-        .value("comment", QPDFTokenizer::token_type_e::tt_comment)
-        .value("inline_image", QPDFTokenizer::token_type_e::tt_inline_image);
-
-    py::class_<QPDFTokenizer::Token>(m, "Token")
-        .def(py::init<QPDFTokenizer::token_type_e, py::bytes>())
-        .def_property_readonly("type_",
-            &QPDFTokenizer::Token::getType,
-            R"~~~(
-                Returns the type of token.
-
-                Return type:
-                    pikepdf.TokenType
-            )~~~")
-        .def_property_readonly("value",
-            &QPDFTokenizer::Token::getValue,
-            R"~~~(
-                Interprets the token as a string.
-
-                Return type:
-                    str or bytes
-            )~~~")
-        .def_property_readonly(
-            "raw_value",
-            [](const QPDFTokenizer::Token &t) -> py::bytes { return t.getRawValue(); },
-            R"~~~(
-                The binary representation of a token.
-
-                Return type:
-                    bytes
-            )~~~")
-        .def_property_readonly("error_msg", &QPDFTokenizer::Token::getErrorMessage)
-        .def("__eq__", &QPDFTokenizer::Token::operator==, py::is_operator());
-
-    py::class_<QPDFObjectHandle::TokenFilter,
-        PointerHolder<QPDFObjectHandle::TokenFilter>>
-        qpdftokenfilter(m, "_QPDFTokenFilter");
-
-    py::class_<TokenFilter, TokenFilterTrampoline, PointerHolder<TokenFilter>>(
-        m, "TokenFilter", qpdftokenfilter)
-        .def(py::init<>())
-        .def("handle_token",
-            &TokenFilter::handle_token,
-            R"~~~(
-                Handle a :class:`pikepdf.Token`.
-
-                This is an abstract method that must be defined in a subclass
-                of ``TokenFilter``. The method will be called for each token.
-                The implementation may return either ``None`` to discard the
-                token, the original token to include it, a new token, or an
-                iterable containing zero or more tokens. An implementation may
-                also buffer tokens and release them in groups (for example, it
-                could collect an entire PDF command with all of its operands,
-                and then return all of it).
-
-                The final token will always be a token of type ``TokenType.eof``,
-                (unless an exception is raised).
-
-                If this method raises an exception, the exception will be
-                caught by C++, consumed, and repalced with a less informative
-                exception. Use :meth:`pikepdf.Pdf.get_warnings` to view the
-                original.
-
-                Return type:
-                    None or list or pikepdf.Token
-            )~~~",
-            py::arg_v("token", QPDFTokenizer::Token(), "pikepdf.Token()"));
 }
